use crate::controllers::{HttpResult, LibVirtReq};
use crate::libvirt_lib_structures::XMLUuid;
use crate::libvirt_rest_structures::nw_filter::NetworkFilter;
use actix_web::{web, HttpResponse};

#[derive(serde::Serialize, serde::Deserialize)]
pub struct NetworkFilterID {
    uid: XMLUuid,
}

/// Get the list of network filters
pub async fn list(client: LibVirtReq) -> HttpResult {
    let networks = match client.get_full_network_filters_list().await {
        Err(e) => {
            log::error!("Failed to get the list of network filters! {e}");
            return Ok(HttpResponse::InternalServerError()
                .json(format!("Failed to get the list of networks! {e}")));
        }
        Ok(l) => l,
    };

    let networks = networks
        .into_iter()
        .map(|n| NetworkFilter::from_xml(n).unwrap())
        .collect::<Vec<_>>();

    Ok(HttpResponse::Ok().json(networks))
}

/// Get the information about a single network filter
pub async fn get_single(client: LibVirtReq, req: web::Path<NetworkFilterID>) -> HttpResult {
    let nwfilter = NetworkFilter::from_xml(client.get_single_network_filter(req.uid).await?)?;
    Ok(HttpResponse::Ok().json(nwfilter))
}
