use crate::libvirt_lib_structures::XMLUuid;
use std::fmt::Display;
use std::net::IpAddr;

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "filterref")]
pub struct NetworkFilterRefXML {
    #[serde(rename(serialize = "@filter"))]
    pub filter: String,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "all")]
pub struct NetworkFilterRuleProtocolAll {}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "mac")]
pub struct NetworkFilterRuleProtocolMac {
    #[serde(
        rename(serialize = "@srcmacaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    srcmacaddr: Option<String>,
    #[serde(
        rename(serialize = "@srcmacmask"),
        skip_serializing_if = "Option::is_none"
    )]
    srcmacmask: Option<String>,
    #[serde(
        rename(serialize = "@dstmacaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    dstmacaddr: Option<String>,
    #[serde(
        rename(serialize = "@dstmacmask"),
        skip_serializing_if = "Option::is_none"
    )]
    dstmacmask: Option<String>,
    #[serde(
        rename(serialize = "@comment"),
        skip_serializing_if = "Option::is_none"
    )]
    comment: Option<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "arp")]
pub struct NetworkFilterRuleProtocolArp {
    #[serde(
        rename(serialize = "@srcmacaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    srcmacaddr: Option<String>,
    #[serde(
        rename(serialize = "@srcmacmask"),
        skip_serializing_if = "Option::is_none"
    )]
    srcmacmask: Option<String>,
    #[serde(
        rename(serialize = "@dstmacaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    dstmacaddr: Option<String>,
    #[serde(
        rename(serialize = "@dstmacmask"),
        skip_serializing_if = "Option::is_none"
    )]
    dstmacmask: Option<String>,
    #[serde(
        rename(serialize = "@arpsrcipaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    arpsrcipaddr: Option<String>,
    #[serde(
        rename(serialize = "@arpsrcipmask"),
        skip_serializing_if = "Option::is_none"
    )]
    arpsrcipmask: Option<u8>,
    #[serde(
        rename(serialize = "@arpdstipaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    arpdstipaddr: Option<String>,
    #[serde(
        rename(serialize = "@arpdstipmask"),
        skip_serializing_if = "Option::is_none"
    )]
    arpdstipmask: Option<u8>,

    #[serde(
        rename(serialize = "@comment"),
        skip_serializing_if = "Option::is_none"
    )]
    comment: Option<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "ipvx")]
pub struct NetworkFilterRuleProtocolIpvx {
    #[serde(
        rename(serialize = "@srcmacaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    srcmacaddr: Option<String>,
    #[serde(
        rename(serialize = "@srcmacmask"),
        skip_serializing_if = "Option::is_none"
    )]
    srcmacmask: Option<String>,
    #[serde(
        rename(serialize = "@dstmacaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    dstmacaddr: Option<String>,
    #[serde(
        rename(serialize = "@dstmacmask"),
        skip_serializing_if = "Option::is_none"
    )]
    dstmacmask: Option<String>,
    #[serde(
        rename(serialize = "@srcipaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    srcipaddr: Option<String>,
    #[serde(
        rename(serialize = "@srcipmask"),
        skip_serializing_if = "Option::is_none"
    )]
    srcipmask: Option<u8>,
    #[serde(
        rename(serialize = "@dstipaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    dstipaddr: Option<String>,
    #[serde(
        rename(serialize = "@dstipmask"),
        skip_serializing_if = "Option::is_none"
    )]
    dstipmask: Option<u8>,

    #[serde(
        rename(serialize = "@comment"),
        skip_serializing_if = "Option::is_none"
    )]
    comment: Option<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "layer4")]
pub struct NetworkFilterRuleProtocolLayer4 {
    #[serde(
        rename(serialize = "@srcmacaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    srcmacaddr: Option<String>,
    #[serde(
        rename(serialize = "@srcipaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    srcipaddr: Option<IpAddr>,
    #[serde(
        rename(serialize = "@srcipmask"),
        skip_serializing_if = "Option::is_none"
    )]
    srcipmask: Option<u8>,
    #[serde(
        rename(serialize = "@dstipaddr"),
        skip_serializing_if = "Option::is_none"
    )]
    dstipaddr: Option<IpAddr>,
    #[serde(
        rename(serialize = "@dstipmask"),
        skip_serializing_if = "Option::is_none"
    )]
    dstipmask: Option<u8>,
    /// Start of range of source IP address
    #[serde(
        rename(serialize = "@srcipfrom"),
        skip_serializing_if = "Option::is_none"
    )]
    srcipfrom: Option<IpAddr>,
    /// End of range of source IP address
    #[serde(
        rename(serialize = "@srcipto"),
        skip_serializing_if = "Option::is_none"
    )]
    srcipto: Option<IpAddr>,
    /// Start of range of destination IP address
    #[serde(
        rename(serialize = "@dstipfrom"),
        skip_serializing_if = "Option::is_none"
    )]
    dstipfrom: Option<IpAddr>,
    /// End of range of destination IP address
    #[serde(
        rename(serialize = "@dstipto"),
        skip_serializing_if = "Option::is_none"
    )]
    dstipto: Option<IpAddr>,
    #[serde(
        rename(serialize = "@srcportstart"),
        skip_serializing_if = "Option::is_none"
    )]
    srcportstart: Option<u16>,
    #[serde(
        rename(serialize = "@srcportend"),
        skip_serializing_if = "Option::is_none"
    )]
    srcportend: Option<u16>,
    #[serde(
        rename(serialize = "@dstportstart"),
        skip_serializing_if = "Option::is_none"
    )]
    dstportstart: Option<u16>,
    #[serde(
        rename(serialize = "@dstportend"),
        skip_serializing_if = "Option::is_none"
    )]
    dstportend: Option<u16>,
    #[serde(rename(serialize = "@state"), skip_serializing_if = "Option::is_none")]
    state: Option<String>,

    #[serde(
        rename(serialize = "@comment"),
        skip_serializing_if = "Option::is_none"
    )]
    comment: Option<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "rule")]
pub struct NetworkFilterRuleXML {
    #[serde(rename(serialize = "@action"))]
    pub action: String,
    #[serde(rename(serialize = "@direction"))]
    pub direction: String,
    #[serde(rename(serialize = "@priority"))]
    pub priority: Option<i32>,

    /// Match all protocols
    #[serde(skip_serializing_if = "Option::is_none")]
    pub all: Option<NetworkFilterRuleProtocolAll>,

    /// Match mac protocol
    #[serde(default, rename = "mac", skip_serializing_if = "Vec::is_empty")]
    pub mac_rules: Vec<NetworkFilterRuleProtocolMac>,

    /// Match arp protocol
    #[serde(default, rename = "arp", skip_serializing_if = "Vec::is_empty")]
    pub arp_rules: Vec<NetworkFilterRuleProtocolArp>,

    /// Match IPv4 protocol
    #[serde(default, rename = "ip", skip_serializing_if = "Vec::is_empty")]
    pub ipv4_rules: Vec<NetworkFilterRuleProtocolIpvx>,

    /// Match IPv6 protocol
    #[serde(default, rename = "ipv6", skip_serializing_if = "Vec::is_empty")]
    pub ipv6_rules: Vec<NetworkFilterRuleProtocolIpvx>,

    /// Match TCP protocol
    #[serde(default, rename = "tcp", skip_serializing_if = "Vec::is_empty")]
    pub tcp_rules: Vec<NetworkFilterRuleProtocolLayer4>,

    /// Match UDP protocol
    #[serde(default, rename = "udp", skip_serializing_if = "Vec::is_empty")]
    pub udp_rules: Vec<NetworkFilterRuleProtocolLayer4>,

    /// Match SCTP protocol
    #[serde(default, rename = "sctp", skip_serializing_if = "Vec::is_empty")]
    pub sctp_rules: Vec<NetworkFilterRuleProtocolLayer4>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "filter")]
pub struct NetworkFilterXML {
    #[serde(rename(serialize = "@name"))]
    pub name: String,
    #[serde(rename(serialize = "@chain"), default)]
    pub chain: String,
    #[serde(
        skip_serializing_if = "Option::is_none",
        rename(serialize = "@priority"),
        default
    )]
    pub priority: Option<i32>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub uuid: Option<XMLUuid>,
    #[serde(default, rename = "filterref", skip_serializing_if = "Vec::is_empty")]
    pub filterrefs: Vec<NetworkFilterRefXML>,
    #[serde(default, rename = "rule", skip_serializing_if = "Vec::is_empty")]
    pub rules: Vec<NetworkFilterRuleXML>,
}

impl NetworkFilterXML {
    pub fn parse_xml<D: Display>(xml: D) -> anyhow::Result<Self> {
        let xml = xml.to_string();

        // We need to put all filter refs at the same location
        let mut filter_refs = Vec::new();
        let xml = lazy_regex::regex_replace_all!(r#"<filterref.*/>"#, &xml, |r: &str| {
            filter_refs.push(r.to_string());

            if r.contains('\n') {
                log::warn!("A filterref contain a new line. This is a symptom of a new unsupported child attribute of <filterref /> object!");
            }

            ""
        });

        let filter_refs = filter_refs.join("\n");
        let xml = xml.replace("</filter>", &format!("{filter_refs}</filter>"));
        log::debug!("Effective NW filter rule parsed: {xml}");

        Ok(serde_xml_rs::from_str(&xml)?)
    }
}
